import {
  DatePicker,
  Descriptions,
  Divider,
  Form,
  Image,
  Input,
  message,
  Modal,
  Select,
  Space,
} from "antd";
import dayjs from "dayjs";
import isBetween from "dayjs/plugin/isBetween";
import { useEffect, useState } from "react";
import { useSelector } from "react-redux";
import axiosInstance, { baseURL } from "../../api/axios";

export default function DeviceDetailModal({ visiable, device, onclose }) {
  const [unavailableTimes, setUnavailableTims] = useState([]);
  const [form] = Form.useForm();
  const userId = useSelector((state) => state.auth.userId);
  const [teams, setTeams] = useState([]);
  const [initialValues, setInitialValues] = useState();

  const fetchTeams = async () => {
    const data = await axiosInstance.get("/teams");
    const teams = data.map((item) => ({
      label: item.label,
      value: item.label,
    }));
    setTeams(teams);
  };

  const fetchUser = async () => {
    const data = await axiosInstance.get(`/userdetail/${userId}`);
    const values = {
      name: data.name,
      phone: data.phone,
      team: data.team,
    };
    setInitialValues(values);
    form.setFieldsValue(values);
  };

  useEffect(() => {
    fetchTeams();
  }, []);

  useEffect(() => {
    const fetchUnavailableTimes = async (id) => {
      const data = await axiosInstance.get(`/device/unavailable-times/${id}`);
      setUnavailableTims(data);
    };

    if (visiable && device?.deviceId) {
      fetchUnavailableTimes(device.deviceId);
      fetchUser();
    }
  }, [visiable, device?.deviceId]);

  const { RangePicker } = DatePicker;
  const disabledDate = (current) => {
    if (!current) return false;
    const today = dayjs().startOf("day");
    const currentDay = current.startOf("day");
    if (currentDay.isBefore(today)) {
      return true;
    }
    dayjs.extend(isBetween);

    return unavailableTimes.some(({ startTime, endTime }) => {
      const start = dayjs(startTime).startOf("day");
      const end = dayjs(endTime).endOf("day");
      return currentDay.isBetween(start, end, null, "[]");
    });
  };

  const handleOK = async () => {
    const values = await form.validateFields();

    const [startTime, endTime] = values.date;

    const payload = {
      deviceId: device.deviceId,
      userId,
      startTime: startTime.format("YYYY-MM-DD"),
      endTime: endTime.format("YYYY-MM-DD"),
      applicantName: values.name,
      applicantContact: values.phone,
      applicantTeam: values.team,
    };

    await axiosInstance.post("/reservation", payload);
    message.success("预约成功");
    form.resetFields();
    onclose();
  };

  return (
    <Modal
      title="预约设备"
      open={visiable}
      onCancel={() => {
        onclose();
      }}
      onOk={handleOK}
    >
      <Space direction="vertical">
        <Descriptions column={1}>
          <Descriptions.Item label="设备名称">{device?.name}</Descriptions.Item>
          <Descriptions.Item label="使用要求">
            {device?.usageRequirement}
          </Descriptions.Item>
          <Descriptions.Item label="设备位置">
            {device?.location}
          </Descriptions.Item>
          <Descriptions.Item label="设备图片">
            <Image src={`${baseURL}/${device?.imagePath}`} />
          </Descriptions.Item>
        </Descriptions>

        <Divider />

        <Form form={form} layout="vertical" initialValues={initialValues}>
          <Form.Item
            name="date"
            label="预约日期"
            rules={[{ required: true, message: "请选择预约日期" }]}
          >
            <RangePicker disabledDate={disabledDate} />
          </Form.Item>
          <Form.Item name="name" label="姓名" rules={[{ required: true }]}>
            <Input />
          </Form.Item>
          <Form.Item name="phone" label="联系电话">
            <Input />
          </Form.Item>
          <Form.Item
            name="team"
            label="所属团队"
            rules={[{ required: true, message: "请选择所属团队" }]}
          >
            <Select options={teams} />
          </Form.Item>
        </Form>
      </Space>
    </Modal>
  );
}
